package org.dasein.cloud.jclouds.gogrid;

import static org.jclouds.concurrent.MoreExecutors.sameThreadExecutor;

import org.dasein.cloud.AbstractCloud;
import org.dasein.cloud.ProviderContext;
import org.dasein.cloud.services.access.AccessServices;
import org.dasein.cloud.services.accounting.AccountingServices;
import org.dasein.cloud.services.address.AddressServices;
import org.dasein.cloud.services.cdn.ContentDistributionServices;
import org.dasein.cloud.services.dc.DataCenterServices;
import org.dasein.cloud.services.disk.SnapshotServices;
import org.dasein.cloud.services.disk.VolumeServices;
import org.dasein.cloud.services.firewall.FirewallServices;
import org.dasein.cloud.services.image.ImageServices;
import org.dasein.cloud.services.lb.LoadBalancerServices;
import org.dasein.cloud.services.scaling.ScalingServices;
import org.dasein.cloud.services.server.Architecture;
import org.dasein.cloud.services.server.Platform;
import org.dasein.cloud.services.server.ServerServices;
import org.dasein.cloud.services.storage.StorageServices;
import org.jclouds.concurrent.config.ExecutorServiceModule;
import org.jclouds.gogrid.GoGridAsyncClient;
import org.jclouds.gogrid.GoGridClient;
import org.jclouds.gogrid.domain.Option;
import org.jclouds.logging.log4j.config.Log4JLoggingModule;
import org.jclouds.rest.RestContext;
import org.jclouds.rest.RestContextFactory;

import com.google.common.collect.ImmutableSet;
import com.google.inject.Module;

public class GoGrid extends AbstractCloud {
    public GoGrid() { }
    
    @Override
    public AccessServices getAccessServices() {
        return null;
    }

    @Override
    public AccountingServices getAccountingServices() {
        return null;
    }

    private GoGridIp addressServices = null;
    
    @Override
    public AddressServices getAddressServices() {
        if( addressServices == null ) {
            addressServices = new GoGridIp(this);
        }
        return addressServices;
    }

    @Override
    public ContentDistributionServices getCdnServices() {
        return null;
    }

    @Override
    public String getCloudName() {
        return "GoGrid";
    }

    RestContext<GoGridClient,GoGridAsyncClient> getCloudClient() {
        ProviderContext ctx = getContext();
        String user = new String(ctx.getAccessPublic());
        String key = new String(ctx.getAccessPrivate());
        
        return (new RestContextFactory()).createContext("gogrid", user, key, ImmutableSet.<Module> of(new Log4JLoggingModule(), new ExecutorServiceModule(sameThreadExecutor(), sameThreadExecutor())));
    }
    
    private transient GoGridDc dataCenterServices = null;
    
    @Override
    public DataCenterServices getDataCenterServices() {
        if( dataCenterServices == null ) {
            dataCenterServices = new GoGridDc(this);
        }
        return dataCenterServices;
    }

    @Override
    public FirewallServices getFirewallServices() {
        return null;
    }

    @Override
    public ImageServices getImageServices() {
        return new GoGridImage(this);
    }

    @Override
    public LoadBalancerServices getLoadBalancerServices() {
        return new GoGridLoadBalancer(this);
    }

    @Override
    public String getProviderName() {
        return "GoGrid";
    }

    @Override
    public ScalingServices getScalingServices() {
        return null;
    }

    @Override
    public ServerServices getServerServices() {
        return new GoGridServer(this);
    }

    @Override
    public SnapshotServices getSnapshotServices() {
        return null;
    }

    @Override
    public StorageServices getStorageServices() {
        return null;
    }

    @Override
    public VolumeServices getVolumeServices() {
        return null;
    }

    Architecture toArchitecture(Option os, String description) {
        Architecture a = toArchitecture(os.getName());
        
        if( a == null && description != null ) {
            a = toArchitecture(description);
        }
        return (a == null ? Architecture.I64 : a);
    }
    
    private Architecture toArchitecture(String name) {
        if( name.contains("_32_") ) {
            return Architecture.I32;
        }
        else if( name.contains("_64_") ) {
            return Architecture.I64;
        }
        else if( name.contains("64") ) {
            return Architecture.I64;
        }
        else if( name.contains("32") ) {
            return Architecture.I32;
        }
        return null;
    }
    
    Platform toPlatform(Option original) {
        if( original == null ) {
            return Platform.UNKNOWN;
        }
        return Platform.guess(original.getName());
    }
}
