package org.dasein.cloud.jclouds.gogrid;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Locale;

import org.dasein.cloud.CloudException;
import org.dasein.cloud.CloudProvider;
import org.dasein.cloud.InternalException;
import org.dasein.cloud.services.dc.DataCenter;
import org.dasein.cloud.services.dc.DataCenterServices;
import org.dasein.cloud.services.dc.Region;
import org.jclouds.gogrid.domain.Option;

public class GoGridDc implements DataCenterServices {
    private GoGrid provider = null;
    
    GoGridDc(GoGrid provider) { 
        this.provider = provider;
    }
    
    @Override
    public DataCenter getDataCenter(String dataCenterId) throws InternalException, CloudException {
        for( DataCenter dc : listDataCenters(provider.getContext().getRegionId()) ) {
            if( dc.getProviderDataCenterId().equals(dataCenterId) ) {
                return dc;
            }
        }
        return null;
    }

    @Override
    public String getProviderTermForDataCenter(Locale locale) {
        return "data center";
    }

    @Override
    public String getProviderTermForRegion(Locale locale) {
        return "region";
    }

    @Override
    public Region getRegion(String regionId) throws InternalException, CloudException {
        if( regionId.equals("1") ) {
            Region region = new Region();
            
            region.setActive(true);
            region.setAvailable(true);
            region.setName("SFO");
            region.setProviderRegionId("1");
            return region;
        }
        return null;
    }

    @Override
    public Collection<DataCenter> listDataCenters(String regionId) throws InternalException, CloudException {
        ArrayList<DataCenter> list = new ArrayList<DataCenter>();
        
        for( Option option : provider.getCloudClient().getApi().getServerServices().getDatacenters() ) {
            DataCenter dc = new DataCenter();
            
            dc.setProviderDataCenterId(String.valueOf(option.getId()));
            dc.setActive(true);
            dc.setAvailable(true);
            dc.setName(option.getName());
            dc.setRegionId(regionId);
            if( dc.getName() == null ) {
                dc.setName(dc.getProviderDataCenterId());
            }
            list.add(dc);
        }
        System.out.println("DC list=" + list);
        return list;
    }

    @Override
    public Collection<Region> listRegions() throws InternalException, CloudException {
        ArrayList<Region> list = new ArrayList<Region>();
        
        list.add(getRegion("1"));
        return list;
    }

    @Override
    public String matchDataCenter(CloudProvider provider, String dataCenterId) throws InternalException, CloudException {
        return (dataCenterId.equals("1") ? "1" : null);
    }
}
